//----------------------------------------------------------------------------
// 
//	file: spikinglayer.h
// 
//	function:  defines a single neural network layer 
//		let's assume the default is 2D, and 1D should be
//		implemented as a column vector (vector[0][n]).
//		derived from class netlayer -- implements spiking network
//
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Tue Jul 21 02:08:23 CDT 1998
// 
//	$Revision: 1.2 $ $Date: 1998/09/04 03:36:37 $
//	$Log: spikinglayer.h,v $
//	Revision 1.2  1998/09/04 03:36:37  yschoe
//	fixed all warnings generated by -Wall
//
//
//----------------------------------------------------------------------------
#ifndef __NETLAYER_H
#define __NETLAYER_H

#include "real.h"
#include "sigmoid.h"
#include "blackboard.h"
#include "spike-code.h"

define MIN_REAL (1.0E-40)

enum NET_NeuronType  {NORMAL, SPIKING};
enum NET_LateralType {SOM, LISSOM};

// structure for Receptive Field (RF) range
typedef struct {
	int left, right; // x range of RF 
	int up, down;	 // y range of RF (not used in 1D map)
} NET_RF_Range;

// structure for weights
typedef struct {
	REAL	**aff_w[];	// afferent wgts (from one or more areas)
  	REAL	**lat_exc_w;	// lateral excitatory weights
	REAL	**lat_inh_w;	// lateral inhibitory weights
} NET_Weights;

// structure for input queues
typedef struct {
	int	**aff_q[];	// afferent incoming queues 
	int 	**lat_exc_q;	// lateral excitatory incoming queues
	int	**lat_inh_q;	// lateral inhibitory incoming queues
	int	*inh_loop;	// inhibitory loop : implements reftactory p.
	REAL	loop_decay;	// decay rate for inh_loop
	REAL	lat_exc_decay;	// decay rate for lat_exc
	REAL	lat_inh_decay;	// decay rate for lat_inh
} NET_Queues;

//----------------------------------------------------------------------------
//      CLASS: netlayer
//		defines a single neural network layer and it's operations
//----------------------------------------------------------------------------
class netlayer {

private:  // private simulation parameters

  // General network parameters
  int	rows, cols;	// layer dimension (size)

  int	aff_rad;	// afferent RF radius
  int	lat_exc_rad;	// lateral excitatory radius
  int	lat_inh_rad;	// lateral inhibitory radius

  REAL	threshold;	// base threshold for firing

  REAL	aff_contri[];	// afferent contribution
  REAL  lat_exc_contri;	// lateral excitatory contribution
  REAL	lat_inh_contri;	// lateral inhibitory contribution

  REAL	aff_noise;	// afferent noise ratio

  REAL	alpha_aff;	// afferent learning rate
  REAL	alpha_lat_exc;	// lateral excitatory learning rate
  REAL  alpha_lat_inh;	// lateral inhibitory learning rate

  // Leaky integrated queue look-up tables
  int   LUT_size;	// LUT size 
  REAL	aff_LUT[]; 	// afferent connection LUT
  REAL	lat_exc_LUT[]; 	// afferent connection LUT
  REAL	lat_inh_LUT[]; 	// afferent connection LUT
  REAL	loop_LUT[]; 	// afferent connection LUT

  // These are 2D matrices, for map layer [x][y].
  REAL		**net_input;	// net input from other layer(s)
  REAL		**output;	// output or q_sum of this layer
  int		**spike;	// spike activity of this layer
  NET_Weights	**wgt;		// weights 
  NET_Queues	**q;		// incoming queues 
  NET_RF_Range  **rf_range[];	// RF range

  // Where the inputs come from (= other layers)
  netlayer 	*from[];	// input from (point to the layer(s))
				// pointer to where the output goes is not 
				// needed -- the receiving end must take care
				// of it.

  // Other administrative members
  blackboard *net_bb;	// blackboard for image dump/load

  inline REAL act_func(REAL netinput); // activation function : inline it

public:

	netlayer(int cols, int rows, 
		 int aff_rad, int exc_rad, int inh_rad, 
		 int from_count,
		 int 
		);
	~netlayer();

  void	activate();
  void	learn();
  display();

};

#endif
